@@ -144,66 +144,73 @@ get_window_for_shape (ThemeImage *image, GdkWindow *window, GtkWidget *widget,
 }
 
 static void
-maybe_check_button_position (GtkWidget *button, ThemeMatchData *match_data)
+check_child_position (GtkWidget      *child,
+		      GList          *children,
+		      ThemeMatchData *match_data)
 {
-  GtkWidget *bbox;
-  gboolean secondary;
-  gboolean horizontal;
-  GtkWidget *first = NULL;
-  GtkWidget *last = NULL;
-  GList *list;
+  GtkWidget *left;
+  GtkWidget *right;
+  GtkWidget *top;
+  GtkWidget *bottom;
+  GList *l;
 
-  bbox = gtk_widget_get_ancestor (button, GTK_TYPE_BUTTON_BOX);
-  if (!bbox)
-    return;
+  left = right = top = bottom = child;
 
-  horizontal = GTK_IS_HBUTTON_BOX (bbox);
-
-  secondary = gtk_button_box_get_child_secondary (GTK_BUTTON_BOX (bbox),
-						  button);
-
-  for (list = GTK_BOX (bbox)->children; list != NULL; list = list->next)
+  for (l = children; l != NULL; l = l->next)
     {
-      GtkBoxChild *child_info = list->data;
-      GtkWidget   *widget = child_info->widget;
+      GtkWidget *widget = l->data;
 
-      if (child_info->is_secondary != secondary ||
-	  !GTK_WIDGET_VISIBLE (widget))
+      if (!GTK_WIDGET_VISIBLE (widget))
 	continue;
+      
+      /* XXX Should we consider the lower right corner instead, for
+       * right/bottom? */
 
-      if (horizontal)
-	{
-	  if (first == NULL || first->allocation.x > widget->allocation.x)
-	    first = widget;
-	  if (last == NULL || last->allocation.x < widget->allocation.x)
-	    last = widget;
-	}
-      else
-	{
-	  if (first == NULL || first->allocation.y > widget->allocation.y)
-	    first = widget;
-	  if (last == NULL || last->allocation.y < widget->allocation.y)
-	    last = widget;
-	}
+      if (left->allocation.x > widget->allocation.x)
+	left = widget;
+      if (right->allocation.x < widget->allocation.x)
+	right = widget;
+      if (top->allocation.y > widget->allocation.y)
+	top = widget;
+      if (bottom->allocation.y < widget->allocation.y)
+	bottom = widget;
     }
 
   match_data->flags |= THEME_MATCH_POSITION;
-  if (horizontal)
+  if (left == child)
+    match_data->position |= THEME_POS_LEFT;
+  if (right == child)
+    match_data->position |= THEME_POS_RIGHT;
+  if (top == child)
+    match_data->position |= THEME_POS_TOP;
+  if (bottom == child)
+    match_data->position |= THEME_POS_BOTTOM;
+}
+
+static void
+check_buttonbox_child_position (GtkWidget *child, ThemeMatchData *match_data)
+{
+  GList *children = NULL;
+  GList *l;
+  GtkWidget *bbox;
+  gboolean secondary;
+
+  g_assert (GTK_IS_BUTTON_BOX (child->parent));
+  bbox = child->parent;
+
+  secondary = gtk_button_box_get_child_secondary (GTK_BUTTON_BOX (bbox), child);
+
+  for (l = GTK_BOX (bbox)->children; l != NULL; l = l->next)
     {
-      match_data->position = THEME_POS_TOP | THEME_POS_BOTTOM;
-      if (first == button)
-	match_data->position |= THEME_POS_LEFT;
-      if (last == button)
-	match_data->position |= THEME_POS_RIGHT;
-    }
-  else
-    {
-      match_data->position = THEME_POS_LEFT | THEME_POS_RIGHT;
-      if (first == button)
-	match_data->position |= THEME_POS_TOP;
-      if (last == button)
-	match_data->position |= THEME_POS_BOTTOM;
+      GtkBoxChild *child_info = l->data;
+      GtkWidget *widget = child_info->widget;
+
+      if (child_info->is_secondary == secondary && GTK_WIDGET_VISIBLE (widget))
+	children = g_list_prepend (children, widget);
     }
+
+  check_child_position (child, children, match_data);
+  g_list_free (children);
 }
 
 static gboolean
@@ -218,6 +225,7 @@ draw_simple_image(GtkStyle       *style,
 		  gint            width,
 		  gint            height)
 {
+  gboolean maemo_position_theming;
   ThemeImage *image;
   
   if ((width == -1) && (height == -1))
@@ -237,9 +245,29 @@ draw_simple_image(GtkStyle       *style,
 	match_data->orientation = GTK_ORIENTATION_HORIZONTAL;
     }
 
-  /* Special handling for buttons in dialogs */
-  if (GTK_IS_BUTTON (widget))
-    maybe_check_button_position (widget, match_data);
+  /* Check for maemo-position-theming to update the position data */
+  if (widget->parent &&
+      gtk_widget_class_find_style_property (GTK_WIDGET_GET_CLASS (widget->parent),
+                                            "maemo-position-theming"))
+    {
+      gtk_widget_style_get (widget->parent,
+			    "maemo-position-theming", &maemo_position_theming,
+			    NULL);
+      if (maemo_position_theming)
+	{
+	  if (GTK_IS_BUTTON_BOX (widget->parent))
+	    check_buttonbox_child_position (widget, match_data);
+	  else
+	    {
+              /* Generic code for other kinds of containers */
+	      GList *children;
+
+	      children = gtk_container_get_children (GTK_CONTAINER (widget->parent));
+	      check_child_position (widget, children, match_data);
+	      g_list_free (children);
+	    }
+	}
+    }
     
   image = match_theme_image (style, match_data);
   if (image)
